pub fn multi_query<'a>(
interval: &GenomicInterval,
genes: &'a AnnotationIndex,
exons: &'a AnnotationIndex,
) -> MultiQueryResult<'a>Expand description
Query multiple annotation indexes simultaneously for better cache locality
This is more efficient than calling query() on each index separately when you need results from multiple sources.